import logging
import os

_logger = logging.getLogger("constants")

HOSTNAME = os.environ.get('SIGNAL_HOST', "localhost")

TELNUMBER = os.environ.get('SIGNAL_NUMBER')
if TELNUMBER is None:
    raise SystemExit("Missing SIGNAL_NUMBER")

POLLING_INTERVAL = int(os.environ.get('SIGNAL_INTERVAL', "15"))

PROTOCOL = os.environ.get('SIGNAL_PROTOCOL', "https")

TEST_MODE = os.environ.get('SIGNAL_TEST')
if TEST_MODE is None:
    TEST_MODE = False
elif TEST_MODE.lower() is "false":
    TEST_MODE = False
else:
    TEST_MODE = True

TEST_DATA = os.environ.get('SIGNAL_TESTDATA', "[]")


def printAllConstants():
    _logger.info("Config: SIGNAL_HOST = %s, SIGNAL_NUMBER = %s, SIGNAL_PROTOCOL = %s,"
                 " SIGNAL_INTERVAL = %s, SIGNAL_TEST = %s, SIGNAL_TESTDATA = %s", HOSTNAME, TELNUMBER, PROTOCOL,
                 POLLING_INTERVAL, TEST_MODE,TEST_DATA)



import json
import logging

import requests

from simple_signal_bot.util.constants import HOSTNAME, TELNUMBER, PROTOCOL, TEST_MODE, TEST_DATA

logger = logging.getLogger("receiver")


def receiveData():
    results = []
    try:
        uri = f"{PROTOCOL}://{HOSTNAME}/v1/receive/{TELNUMBER}"
        logger.debug("Fetching Signal-CLI updates from %s", uri)
        response = requests.get(uri, timeout=120)
        logger.debug("Parsing JSON Response")
        results = response.json()
        logger.debug("Fetched update: %s", results)
    except ValueError as vexc:
        logger.error("ValueError: %s", vexc)
    except requests.exceptions.RequestException as e:
        logger.error("RequestException: %s", e)

    return results


if TEST_MODE:
    logger.info("Overriding recieveData with Test Data")


    def receiveData():
        logger.info("This is Test Data")
        try:
            results = json.loads(TEST_DATA)
            logger.debug("Fetched update: %s", results)
        except json.decoder.JSONDecodeError as jse:
            logger.error("Test Data JSONDecodeError: %s", jse)
            return []
        return results



import logging

from simple_signal_bot.manager.regex_manager import RegexHandler

logger = logging.getLogger("MessageDecorator")


class MessageDecorator:

    def __init__(self, regex_manager):
        self.regex_manager = regex_manager
        pass

    def decorator_factory(self, regex=".*"):
        def decorator(function):
            handler = RegexHandler(function, regex)
            self.regex_manager.register_handler(handler)
            logger.info("Hooked a MessageDecorator for %s", regex)
            return function

        return decorator



import logging
import time
from datetime import timedelta
from timeloop import Timeloop

from simple_signal_bot.decorator.command_decorator import CommandDecorator
from simple_signal_bot.decorator.message_decorator import MessageDecorator
from simple_signal_bot.manager.regex_manager import RegexManager
from simple_signal_bot.signal_io.receiver import receiveData
from simple_signal_bot.signal_io.sender import Sender
from simple_signal_bot.util.constants import POLLING_INTERVAL, printAllConstants
from simple_signal_bot.util.envelope_utils import getSourceInformation


class SignalBot:

    def __init__(self):
        self.update_decorators = list()
        self.logger = logging.getLogger("core")
        regex_manager = RegexManager()
        self.__registerManager(regex_manager)
        self.Message = MessageDecorator(regex_manager).decorator_factory
        self.Command = CommandDecorator(regex_manager).decorator_factory
        self.sender = Sender()

    def __process_envelope(self, envelope):
        source_information = getSourceInformation(envelope)
        if "dataMessage" in envelope:
            data_message = envelope["dataMessage"]
            for x in range(len(self.update_decorators)):
                if self.update_decorators[x].check_update(source_information, data_message):
                    self.update_decorators[x].process_update(source_information, data_message)
        else:  # Updates ohne dataMessage sind sync/call/receipt/typing und werden daher ignoriert
            pass

    def __registerManager(self, manager):
        self.logger.info("Registered Decorator Manager %s", manager)
        self.update_decorators.append(manager)

    def run(self):
        printAllConstants()

        # Start Core Loop
        tl = Timeloop()
        logging.getLogger("timeloop").setLevel(logging.CRITICAL)
        self.logger.info("Polling every %i seconds", POLLING_INTERVAL)

        @tl.job(interval=timedelta(seconds=POLLING_INTERVAL))
        def receive_polling():
            self.logger.debug("Fetching updates")
            results = receiveData()
            for x in range(len(results)):
                delivery = results[x]
                if "envelope" in delivery:
                    envelope = delivery["envelope"]
                    self.__process_envelope(envelope)
            return False

        tl.start()

        while True:
            try:
                time.sleep(1)
            except KeyboardInterrupt:
                tl.stop()
                break



import logging

import requests

from simple_signal_bot.util.constants import TELNUMBER, HOSTNAME, PROTOCOL, TEST_MODE

logger = logging.getLogger("sender")


def post(uri, body):
    logger.debug("Sending message via %s", uri)
    post_response = requests.post(uri, json=body, timeout=60)
    logger.debug("Response: %s", post_response)


if TEST_MODE:
    logger.info("Catching outgoing messages")


    def post(uri, body):
        logger.debug("Sending message via %s", uri)
        logger.info("Sending TEST uri: %s body: %s", uri, body)
        logger.debug("Response: %s", "Tis but a tests")


class Sender:
    @staticmethod
    def sendText(recepient, message):
        body = {
            "message": f"{message}",
            "number": f"{TELNUMBER}",
            "recipients": [
                f"{recepient}"
            ]
        }
        uri = f"{PROTOCOL}://{HOSTNAME}/v2/send"
        logger.debug("Sending message '%s' to %s", message, recepient)
        post(uri, body)



from simple_signal_bot.util.group_utils import groupContextFromId


def getOptionalGroupContext(data_message):
    if "groupInfo" in data_message:
        internal = data_message["groupInfo"]["groupId"]
        return groupContextFromId(internal)
    return None



import logging

from simple_signal_bot.manager.regex_manager import RegexHandler

logger = logging.getLogger("CommandDecorator")


class CommandDecorator:

    def __init__(self,regex_manager):
        self.regex_manager = regex_manager
        pass

    def decorator_factory(self, cmd=None):
        def decorator(function):
            def newfunc(source_information, data_message, context, message):
                args = str(message).split(" ")
                function(source_information, data_message, context, message, args)
            regex = f"^/{cmd}\\s*"
            if cmd is None:
                regex = f"^/{function.__name__}\\s*"
            handler = RegexHandler(newfunc, regex)
            self.regex_manager.register_handler(handler)
            logger.info("Hooked a CommandDecorator for %s", regex)
            return newfunc

        return decorator



import logging

from simple_signal_bot.util.context import getOptionalGroupContext
from simple_signal_bot.util.regex import regexCheck

logger = logging.getLogger("RegexManager")


class RegexHandler:
    def __init__(self, callback, regex):
        self.callback=callback
        self.regex = regex


class RegexManager:
    def __init__(self):
        self.regex_handlers = list()
        pass

    def register_handler(self,regex_handler):
        self.regex_handlers.append(regex_handler)

    def process_update(self, source_information, data_message):
        logger.debug("Processing update")
        if "message" in data_message:
            msg = data_message["message"]
            if msg is not None:
                for x in range(len(self.regex_handlers)):
                    handler = self.regex_handlers[x]
                    if regexCheck(handler.regex, msg):
                        logger.debug("Message '%s' fit '%s'", msg, handler.regex)
                        context = getOptionalGroupContext(data_message)
                        if context is None:
                            context = source_information.source_uuid
                        handler.callback(source_information, data_message, context, msg)
                    else:
                        logger.debug("Message '%s' didn't fit '%s'", msg, handler.regex)
            else:
                logger.debug("Empty message")
        else:
            logger.debug("Missing message")

    @staticmethod
    def check_update(source_information, data_message):
        if "message" in data_message:
            logger.debug("Handler check passed")
            msg = data_message["message"]
            return msg is not None
        return False



from simple_signal_bot.model.source_information import SourceInformation


def getSourceInformation(envelope):
    return SourceInformation(envelope["sourceUuid"],
                             envelope["sourceNumber"],
                             envelope["sourceName"],
                             envelope["sourceDevice"])



class SourceInformation:
    def __init__(self, source_uuid, source_number, source_name, source_device):
        self.source_uuid = source_uuid
        self.source_number = source_number
        self.source_name = source_name
        self.source_device = source_device



import logging
import re

logger = logging.getLogger("regex")


def regexCheck(regex, msg):
    logger.debug("Using RegEx %s on message %s", regex, msg)
    try:
        pattern = re.compile(regex)
        success = pattern.match(msg)
        logger.debug("RegEx returned %s", success)
        return success

    except re.error:
        logger.error("Invalid RegEx: %s", regex)
    return False



import base64


def groupContextFromId(group_id):  # Sollte diese Methode nicht mehr gehen so muss über v1/groups die id abgeglichen werden
    return f"group.{base64.b64encode(group_id.encode('ascii')).decode('ascii')}"